// BasicEvent.cpp: implementation of the CBasicEvent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Player.h"
#include "Event.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBasicEvent::CBasicEvent(EEventType eType, bool bTemp) : m_eEventType(eType)
{
	if (bTemp)
		m_nRefCount = 0;
	else
		m_nRefCount = 1;
}

CBasicEvent::~CBasicEvent()
{
	TRACE("~CBasicEvent: %d\n", m_eEventType);
}

long CBasicEvent::AddRef()
{
	return InterlockedIncrement(&m_nRefCount);
}

long CBasicEvent::Release()
{
	long nNewCount = InterlockedDecrement(&m_nRefCount);
	ASSERT(nNewCount >= 0);
	if (!nNewCount)
		delete this;
	return nNewCount;
}

//////////////////////////////////////////////////////////////////////
// CConnectEvent Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CConnectEvent::CConnectEvent(string sHost, unsigned nPort) :
	CBasicEvent(ET__SRVCONNECT, true), m_sHost(sHost), m_nPort(nPort)
{
}

CConnectEvent::~CConnectEvent()
{
}

//////////////////////////////////////////////////////////////////////
// CSendEvent Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSendEvent::CSendEvent(CServerMsg *pSrvMsg) :
	CBasicEvent(ET__SEND_SRVMSG, true), m_pSrvMsg(pSrvMsg)
{
}

CSendEvent::CSendEvent(string sMessage) :
    CBasicEvent(ET__SEND_SRVMSG, true), m_pSrvMsg(new CServerMsg(sMessage))
{
}

CSendEvent::~CSendEvent()
{
    if (m_pSrvMsg) {
        delete m_pSrvMsg;
        m_pSrvMsg = NULL;
    }
}

//////////////////////////////////////////////////////////////////////
// CReceiveEvent Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CReceiveEvent::CReceiveEvent(CServerMsg *pSrvMsg) :
	CBasicEvent(ET__RECEIVE_SRVMSG, true), m_pSrvMsg(pSrvMsg)
{
}

CReceiveEvent::CReceiveEvent(string sMessage, TSystemTime tReceivedTime) :
    CBasicEvent(ET__RECEIVE_SRVMSG, true), m_pSrvMsg(new CServerMsg(sMessage, tReceivedTime))
{
}

CReceiveEvent::~CReceiveEvent()
{
    if (m_pSrvMsg) {
        delete m_pSrvMsg;
        m_pSrvMsg = NULL;
    }
}

//////////////////////////////////////////////////////////////////////
// CReceiveEvent Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CProcessPerceptionEvent::CProcessPerceptionEvent(CPerception *pPerception) :
    CBasicEvent(ET__PROCESS_PERCEPTION, true), m_pPerception(pPerception)
{
}

CProcessPerceptionEvent::~CProcessPerceptionEvent()
{
    if (m_pPerception) {
        delete m_pPerception;
        m_pPerception = NULL;
    }
}

//////////////////////////////////////////////////////////////////////
// CSendCommandEvent Class
//////////////////////////////////////////////////////////////////////

CSendCommandEvent::CSendCommandEvent(CGameCommand *pGameCommand) :
    CBasicEvent(ET__SEND_COMMAND, true), m_pGameCommand(pGameCommand)
{
}

CSendCommandEvent::~CSendCommandEvent()
{
    if (m_pGameCommand) {
        delete m_pGameCommand;
        m_pGameCommand = NULL;
    }
}

//////////////////////////////////////////////////////////////////////
// CDeliberateEvent Class
//////////////////////////////////////////////////////////////////////

CDeliberateEvent::CDeliberateEvent(CGameState &cGameState, CPerceptionSeq *pPerceptionSeq, CGameCommandSeq *pSentCommandSeq) :
    CBasicEvent(ET__DELIBERATE, true), m_pPerceptionSeq(pPerceptionSeq),
    m_pSentCommandSeq(pSentCommandSeq)
{
    m_cGameState = cGameState;
}

CDeliberateEvent::~CDeliberateEvent()
{
    if (m_pPerceptionSeq != NULL) {
        delete m_pPerceptionSeq;
        m_pPerceptionSeq = NULL;
    }
    if (m_pSentCommandSeq != NULL) {
        delete m_pSentCommandSeq;
        m_pSentCommandSeq = NULL;
    }
}

//////////////////////////////////////////////////////////////////////
// CActEvent Class
//////////////////////////////////////////////////////////////////////

CActEvent::CActEvent(CGameCommandSeq *pGameCommandSeq) :
    CBasicEvent(ET__ACT, true), m_pGameCommandSeq(pGameCommandSeq)
{
}

CActEvent::~CActEvent()
{
    if (m_pGameCommandSeq) {
        delete m_pGameCommandSeq;
        m_pGameCommandSeq = NULL;
    }
}

//////////////////////////////////////////////////////////////////////
// CNewAgentEvent Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNewAgentEvent::CNewAgentEvent(CAgent* pAgent) : CBasicEvent(ET__NEWAGENT, true)
{
	m_pAgent = pAgent;
}

CNewAgentEvent::CNewAgentEvent(const CNewAgentEvent& pNewAgentEvent) : CBasicEvent(pNewAgentEvent)
{
	m_pAgent = new CAgent(*pNewAgentEvent.m_pAgent);
}

CNewAgentEvent::~CNewAgentEvent()
{
	if (m_pAgent)
		delete m_pAgent;
}

//////////////////////////////////////////////////////////////////////
// CNewPerception Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNewPerceptionEvent::CNewPerceptionEvent(CPerception* pPerception) :
	CBasicEvent(ET__NEWPERCEPTION, true)
{
	m_pPerception = pPerception;
}

CNewPerceptionEvent::CNewPerceptionEvent(const CNewPerceptionEvent& pNewPerceptionEvent) :
	CBasicEvent(pNewPerceptionEvent)
{
	m_pPerception = new CPerception(*pNewPerceptionEvent.m_pPerception);
}

CNewPerceptionEvent::~CNewPerceptionEvent()
{
	if (m_pPerception)
		delete m_pPerception;
}
