// BasicEvent.h: interface for the CBasicEvent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASICEVENT_H__759E8308_CED7_11D4_BEC2_00C0CA122103__INCLUDED_)
#define AFX_BASICEVENT_H__759E8308_CED7_11D4_BEC2_00C0CA122103__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma warning (disable : 4786)

#include "ServerMsg.h"
#include "GameState.h"
#include "Perception.h"
#include "GameCommand.h"
#include <string>
#include "worldobjects.h"
#include "Perception.h"

using namespace std;

#define COPY_RELEASE(T) \
public: \
virtual CBasicEvent* CopyRelease() \
{ \
	long nNewCount = InterlockedDecrement(&m_nRefCount); \
	if (nNewCount) \
		return new T(*this); \
	else \
		return this; \
}

enum EEventType { 
    ET__SRVCONNECT,
    ET__SRVDISCONNECT,
    ET__RECEIVE_SRVMSG,
    ET__SEND_SRVMSG,
    ET__PROCESS_PERCEPTION,
    ET__SEND_COMMAND,
    ET__DELIBERATE,
    ET__ACT,
	ET__NEWPERCEPTION,
	ET__NEWAGENT
};

class CBasicEvent  
{
public:
	CBasicEvent(EEventType eType, bool bTemp = false);
	virtual ~CBasicEvent();
	
	long AddRef();
	long Release();

	EEventType m_eEventType;

	COPY_RELEASE(CBasicEvent);

protected:
	long m_nRefCount;
};


class CConnectEvent : public CBasicEvent  
{
public:
	string m_sHost;
	unsigned m_nPort;

public:
	CConnectEvent(string sHost, unsigned nPort);
	virtual ~CConnectEvent();

	COPY_RELEASE(CConnectEvent);
};

class CSendEvent : public CBasicEvent  
{
public:
    CServerMsg *m_pSrvMsg;

public:
	CSendEvent(CServerMsg *pSrvMsg);
    CSendEvent(string sMessage);
	virtual ~CSendEvent();

	COPY_RELEASE(CSendEvent);
};

class CReceiveEvent : public CBasicEvent  
{
public:
    CServerMsg *m_pSrvMsg;

public:
	CReceiveEvent(CServerMsg *pSrvMsg);
    CReceiveEvent(string sMessage, TSystemTime tReceivedTime);
	virtual ~CReceiveEvent();

	COPY_RELEASE(CReceiveEvent);
};

class CProcessPerceptionEvent : public CBasicEvent
{
public:
    CPerception *m_pPerception;

public:
	CProcessPerceptionEvent(CPerception *pPerception);
	virtual ~CProcessPerceptionEvent();

	COPY_RELEASE(CProcessPerceptionEvent);
};

class CSendCommandEvent : public CBasicEvent
{
public:
    CGameCommand *m_pGameCommand;

public:
	CSendCommandEvent(CGameCommand *pGameCommand);
	virtual ~CSendCommandEvent();

	COPY_RELEASE(CSendCommandEvent);
};

class CDeliberateEvent : public CBasicEvent
{
public:
    // +DeliberateTime;
    CGameState m_cGameState;
    CPerceptionSeq *m_pPerceptionSeq;
    CGameCommandSeq *m_pSentCommandSeq;

public:
	CDeliberateEvent(CGameState &cGameState, CPerceptionSeq *pPerceptionSeq, CGameCommandSeq *pSentCommandSeq);
	virtual ~CDeliberateEvent();

	COPY_RELEASE(CDeliberateEvent);
};

class CActEvent : public CBasicEvent
{
public:
    CGameCommandSeq *m_pGameCommandSeq;

public:
	CActEvent(CGameCommandSeq *pGameCommandSeq);
	virtual ~CActEvent();

	COPY_RELEASE(CActEvent);
};

class CNewAgentEvent : public CBasicEvent  
{
public:
	CNewAgentEvent(CAgent* pAgent);
	CNewAgentEvent(const CNewAgentEvent& pNewAgent);
	virtual ~CNewAgentEvent();

	CAgent* m_pAgent;

};

class CNewPerceptionEvent : public CBasicEvent  
{
public:
	CNewPerceptionEvent(CPerception* pPerception);
	CNewPerceptionEvent(const CNewPerceptionEvent& pNewPerceptionEvent);
	virtual ~CNewPerceptionEvent();

	CPerception* m_pPerception;
};

#endif // !defined(AFX_BASICEVENT_H__759E8308_CED7_11D4_BEC2_00C0CA122103__INCLUDED_)
