#include "stdafx.h"
#include "ServerCommunicator.h"

CServerCommunicator::CServerCommunicator(CWinThread* pDefThread, CCoordinator* pCoordinator, int SP_SIMULATOR_STEP, CSystemClock *pSystemClock) : 
    CEventRecipient(pDefThread, pCoordinator), m_pSystemClock(pSystemClock)

{
    m__SP_SIMULATOR_STEP = SP_SIMULATOR_STEP;
    m_pPerceptionSeq = NULL;
	m_pSentCommandSeq = NULL;
    
    RegisterEvent(ET__PROCESS_PERCEPTION);
    RegisterEvent(ET__ACT);
}

CServerCommunicator::~CServerCommunicator()
{
    UnregisterEvent(ET__PROCESS_PERCEPTION);
    UnregisterEvent(ET__ACT);

    if (m_pPerceptionSeq != NULL)
        delete m_pPerceptionSeq;

    if (m_pSentCommandSeq != NULL)
        delete m_pSentCommandSeq;
}

/* pre zadany systemovy cas upravi hraci cas GameState.GameTime 
   na zaklade hodnot m_nSynchPlayTime a m_nSynchSystemTime */
/* upravuje cas iba smerom nahor => cas musi narastat, preto nie je mozne
   spracovat po volani tejto funkcie starsie vnemy, ktore prisli pred nou
   (automaticky sa im priradi hodnota naposledy zvacseneho casu) */
int CServerCommunicator::UpdateGameTime(TSystemTime tSystemTime)
{
    TSystemTime tSynchTime;
    int nPlayTime, nStopCycles;

    if ((!m_bSynchronized) || (m_tLastSystemTime > tSystemTime)) return -1;

    // vypocitame nSynchTime = ako dlho sme zosynchronizovany
    // skontrolujeme, ci narasta systemovy cas -> inak sme rozsynchronizovany
    tSynchTime = tSystemTime - m_tSynchSystemTime;
    if (tSynchTime < 0) { m_bSynchronized = FALSE; return -1; }

    m_tLastSystemTime = tSystemTime;

    // vypocitame nove hodnoty herneho cas
    if (!isStopMode(m_cGameState.GetPlayMode()))
    {
        // FIXME - pre Windowsacky soccer server nebudeme umelo zvysovat cas
        //  => cas sa zvysuje len podla prijatych sprav
#ifdef BLIND_TIME_COUNT
        nPlayTime = m_nSynchPlayTime + (nSynchTime / SP_SEND_STEP);
#else
        nPlayTime = m_nSynchPlayTime;
#endif
        nStopCycles = 0;
    } else {
        nPlayTime = m_nSynchPlayTime;
#ifdef SENSEBODY_STOPTIME_COUNT
        nStopCycles = nSenseBodyCnt??;   // FIXME: neimplementovane
#else
        nStopCycles = tSynchTime / m__SP_SIMULATOR_STEP;
#endif
    }

    // nastavime novu hodnotu a urcime celkovy pocet cyklov
    CGameTime tGameTime = m_cGameState.GetGameTime();
    if (tGameTime.m_nPlayTime == nPlayTime) {
        tGameTime.m_nAllCycles += nStopCycles - tGameTime.m_nStopCycles;
    } else {
        tGameTime.m_nAllCycles += nPlayTime - tGameTime.m_nPlayTime;
    }
    tGameTime.m_nPlayTime = nPlayTime;
    tGameTime.m_nStopCycles = nStopCycles;

    m_cGameState.SetGameTime(tGameTime);

    //fprintf(m_ff, "Cas: %d, %d, %d (%d)\n", tGameTime.m_nPlayTime, tGameTime.m_nStopCycles, tGameTime.m_nAllCycles, nSynchTime);

    return 0;
}

/* upravi cas synchronizacie na zaklade casu prichodu prvej udalosti 
   so zmenenym hracim casom */
void CServerCommunicator::UpdateSynchTime(int nPlayTime, int tSystemTime)
{
    // nastavim zaciatok noveho sim. cyklu podla prvej spravy s novym nPlayTime
    //   FIXME?: predpokladame, ze sense_body pride vzdy na zaciatku cyklu
    //     nemusi platit v pripade, ze sami vysleme ziadost o sense_body
    if ((!m_bSynchronized) || (m_nSynchPlayTime != nPlayTime)) {
        // ak si myslim, ze som v stopmode, ale zmenil sa cas (=nie hned po inicializacii), tak asi nemam 
        //   aktualne informacie -> nastavim, ze playmod je nejaky nestopmodovy
        if ((m_nSynchPlayTime > 0) && (isStopMode(m_cGameState.GetPlayMode()))) {
            m_cGameState.SetPlayMode(PM_UNKNOWN_NOSTOP);
        }

        // nastavime, ze v case nSystemTime nastal novy sim. krok nPlayTime
        m_nSynchPlayTime = nPlayTime;
        m_tSynchSystemTime = tSystemTime;
        m_bSynchronized = TRUE;
    }

    UpdateGameTime(tSystemTime);
}

void CServerCommunicator::UpdateGameState(CPerception *pPerception)
{
    EPlayMode nPlayMode;

    // nastavime playmod podla perceptionu 
    nPlayMode = pPerception->GetPlayMode();
    if (isValidPlayMode(nPlayMode)) {
        m_cGameState.SetPlayMode(nPlayMode);
    }

    // uprav cas synchronizacie
    UpdateSynchTime(pPerception->GetPlayTime(), pPerception->GetReceivedTime());

//  FIXME
//  if ((m_pGameState->CurrentTime - oldTime) > 1) && (...)) {
//      error("missed simulation step");
//  }
}

void CServerCommunicator::ProcessReceivedSrvMsg(CPerception *pPerception)
{
    // updejtneme aktualny GameState
    UpdateGameState(pPerception);

    // ak pride sensebody tak skontroluj, ci vykonal posledny command
    // inak ho zmaz zo SentCommandsSeq

    // nastavime aktualny GameTime pre vnem
    //   FIXME?: predpokladam, ze perception sa spracuje este v tom cykle,
    //   kedy pride -> je potrebne spracuvat v realnom case
    pPerception->SetGameTime(m_cGameState.GetGameTime());

    // pridame vnem do postupnosti, ak nemame postupnost, tak vytvorime
    if (m_pPerceptionSeq == NULL)
        m_pPerceptionSeq = new CPerceptionSeq();
    m_pPerceptionSeq->AddPerception(pPerception);
}

void CServerCommunicator::OnSynchTimer(int nTimeToNextCall /*= 0*/)
{   
    int nStepTime;                  // ako dlho uz trva aktualny sim. cyklus
    int nGameCycles = m_cGameState.GetGameTime().m_nAllCycles;

    nStepTime = UpdateGameTime(m_pSystemClock->GetTime());
    if (nStepTime < 0) return;

    // FIXME - napevno: raz za simulacny krok rozmyslaj a posli akciu
    if (m_nLastDeliberateTime != nGameCycles) {
//  if (nStepTime >= SP_SEND_STEP - TIME_TO_DELIBERATE - TIME_TO_SEND) {
        TRACE("rozmyslaj\n");
        m_nLastDeliberateTime = nGameCycles;

		// nemozeme poslat NULLovy m_pSentCommandSeq
        if (m_pSentCommandSeq == NULL)
            m_pSentCommandSeq = new CGameCommandSeq();
        PostEvent(new CDeliberateEvent(m_cGameState, m_pPerceptionSeq, m_pSentCommandSeq));

        // postupnosti sme poslali eventom, takze uz ich nesmieme pouzivat
        m_pPerceptionSeq = NULL;  m_pSentCommandSeq = NULL;
    }
}

void CServerCommunicator::OnEvent(CBasicEvent *pEvent)
{
	TRACE("OnEvent: %d\n", pEvent->m_eEventType);

	switch (pEvent->m_eEventType)
	{
	case ET__PROCESS_PERCEPTION:
        CPerception *pPerception;

        // FIXME: potrebujem urobit copy-release
        pPerception = new CPerception(*((CProcessPerceptionEvent*) pEvent)->m_pPerception);

        ProcessReceivedSrvMsg(pPerception);
        OnSynchTimer(0);      
		break;

	case ET__ACT:
        CGameCommand *pSendCommand0, *pSendCommand;
        CGameCommandSeq *pSendCommandSeq;

        pSendCommandSeq = ((CActEvent*) pEvent)->m_pGameCommandSeq;
        while (pSendCommandSeq->GetGameCommand(&pSendCommand0))
        {
            // skontrolujeme, ci je povel urceny pre aktualny sim. krok
            if (m_cGameState.GetGameTime() == pSendCommand0->GetGameTime()) 
            {
                // potrebujem jednu kopiu
                pSendCommand = new CGameCommand(*pSendCommand0);
                PostEvent(new CSendCommandEvent(pSendCommand));

                if (m_pSentCommandSeq == NULL)
                    m_pSentCommandSeq = new CGameCommandSeq();
                m_pSentCommandSeq->AddGameCommand(pSendCommand0);
            } else delete pSendCommand0;
        }
		break;
	}

	pEvent->Release();
}
