// VisPerception.cpp: implementation of the CVisPerception class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Player.h"
#include "VisPerception.h"
#include "WorldObjects.h"
#include "SoccerAgent.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVisPerception::CVisPerception(CWinThread* pDefThread, CCoordinator* pCoordinator) : CEventRecipient(pDefThread, pCoordinator)
{
	RegisterEvent(ET__NEWAGENT);
	RegisterEvent(ET__NEWPERCEPTION);

	m_pPerception = NULL;
	m_pAgent = NULL;
}

CVisPerception::~CVisPerception()
{
	UnregisterEvent(ET__NEWAGENT);
	UnregisterEvent(ET__NEWPERCEPTION);

	if (m_pPerception)
		delete m_pPerception;
	if (m_pAgent)
		delete m_pAgent;
}

void CVisPerception::SetPerception(CPerception* newPerception)
{
	if (m_pPerception)
		delete m_pPerception;
	m_pPerception = newPerception;
}

void CVisPerception::SetAgent(CAgent* newAgent)
{
	if (m_pAgent)
		delete m_pAgent;
	m_pAgent = newAgent;
}

//////////////////////////////////////////////////////////////////////////////
// int CVisPerception::GetSeenFlag( int index, CVisObject* visObject )
// ---------------------------------------------------------------------------
// Description:
//		
//
// ---------------------------------------------------------------------------
// Parameters:
//		int index
//		CVisObject* visObject
//
//////////////////////////////////////////////////////////////////////////////
int CVisPerception::GetSeenFlag( int index, CVisObject* visObject )
{
	CSeenFlag	flag;
	int			retVal = m_pPerception->GetNextFlag( &flag, index );

	if ( retVal == -1 )
		return retVal;

	visObject->CreateFlag( &flag, m_pAgent );
	return retVal;
}

//////////////////////////////////////////////////////////////////////////////
// int CVisPerception::GetSeenPlayer( int index, CVisObject* visObject )
// ---------------------------------------------------------------------------
// Description:
//		
//
// ---------------------------------------------------------------------------
// Parameters:
//		int index
//		CVisObject* visObject
//
//////////////////////////////////////////////////////////////////////////////
int CVisPerception::GetSeenPlayer( int index, CVisObject* visObject )
{
	CSeenPlayer	player;
	int			retVal = m_pPerception->GetNextPlayer( &player, index );

	if ( retVal == -1 )
		return retVal;

	// the same for teammate and opponent ???????

	visObject->CreatePlayer( &player, m_pAgent );
	return retVal;
}

//////////////////////////////////////////////////////////////////////////////
// int CVisPerception::GetSeenBall( CVisObject* visObject )
// ---------------------------------------------------------------------------
// Description:
//		
//
// ---------------------------------------------------------------------------
// Parameters:
//		CVisObject* visObject
//
//////////////////////////////////////////////////////////////////////////////
int CVisPerception::GetSeenBall( CVisObject* visObject )
{
	CSeenObject	object;
	bool		pRetVal = m_pPerception->GetBall( &object );

	if ( !pRetVal )
		return -1;

	visObject->CreateBall( &object, m_pAgent );
	return 1;
}

void CVisPerception::OnEvent(CBasicEvent *pEvent)
{
	TRACE("OnEvent: %d\n", pEvent->m_eEventType);

	switch (pEvent->m_eEventType)
	{
	case ET__NEWAGENT:
		{
		CNewAgentEvent* pNewAgentEvent = (CNewAgentEvent*) pEvent->CopyRelease();
		SetAgent(pNewAgentEvent->m_pAgent);
		pNewAgentEvent->m_pAgent = NULL;
		delete pNewAgentEvent;
		break;
		}
	case ET__NEWPERCEPTION:
		{
		CNewPerceptionEvent* pNewPerceptionEvent = (CNewPerceptionEvent*) pEvent->CopyRelease();
		SetPerception(pNewPerceptionEvent->m_pPerception);
		pNewPerceptionEvent->m_pPerception = NULL;
		delete pNewPerceptionEvent;
		break;
		}
	}

}